import logging
from abc import abstractmethod

logger = logging.getLogger(__name__)


class ACDLChunkLoadStateTupleABC:
    """ Agent Import Chunk

    This table stores information used by the agent about when things need updating.
    There is no relation to the server objects, the agent can use this how ever it wants.

    The server never manipulates this table it's self.

    """

    @classmethod
    @abstractmethod
    def sqlCoreIdColumn(cls):
        """ SQL Core - Chunk Key Column

        This method should return the SQL Core column

        :return: The column

        Example code:

                return cls.__table__.c.id

        """
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def sqlCoreLoad(cls, row):
        """ SQL Core - Load

        This method creates the declarative from an SQL core query.
        There is a lot of overhead from using the ORM to load this
        then expunging it. So instead an SQLCore query then create the tuple.

        Example code:

            return GraphSegmentLoadStateTuple(id=row.id,
                                            chunkKey=row.chunkKey,
                                            lastImportDate=row.lastImportDate,
                                            lastImportHash=row.lastImportHash)
        """
        raise NotImplementedError()



from peek_plugin_base.storage.DbConnection import DbSessionCreator
from peek_abstract_chunked_data_loader.private.server.admin_backend.AppSettingHandler import \
    makeAppSetttingsHandler
from peek_abstract_chunked_data_loader.private.server.admin_backend.CustomAttrTableHandler import \
    makeCustomAttrTableHandler
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

from peek_abstract_chunked_data_loader.private.server.admin_backend.CustomHeaderTableHandler import \
    makeCustomHeaderTableHandler
from .SettingPropertyHandler import makeSettingPropertyHandler


def makeAdminBackendHandlers(dbSessionCreator: DbSessionCreator,
                             observable: TupleDataObservableHandler):
    yield makeAppSetttingsHandler(dbSessionCreator)

    yield makeSettingPropertyHandler(dbSessionCreator)

    yield makeCustomAttrTableHandler(observable, dbSessionCreator)
    yield makeCustomHeaderTableHandler(observable, dbSessionCreator)



import logging
from abc import ABCMeta
from typing import Optional

from peek_abstract_chunked_data_loader.private.tuples.ACDLChunkLoadStateTupleABC import \
    ACDLChunkLoadStateTupleABC
from peek_plugin_base.storage.LoadPayloadPgUtil import getTuplesPayloadBlocking
from peek_storage.plpython.LoadPayloadPgUtil import LoadPayloadTupleResult
from sqlalchemy import select
from sqlalchemy.sql import Select

logger = logging.getLogger(__name__)


class ACDLRpcForAgentImportABC(metaclass=ABCMeta):
    _StateTupleDeclarative: ACDLChunkLoadStateTupleABC = None

    def __init__(self, dbSessionCreator):
        self._dbSessionCreator = dbSessionCreator

    # -------------
    def adlInitialLoadOfStatePayloadBlocking(self, offset: int,
                                             count: int,
                                             sql: Optional[Select] = None
                                             ) -> Optional[bytes]:
        """ Chunked Key Index - Initial Load Chunks Blocking

        This method is used to load the initial set of chunks from the server
        to the client.

        """
        if sql is None:
            table = self._StateTupleDeclarative.__table__
            sql = select([table]) \
                .order_by(self._StateTupleDeclarative.sqlCoreIdColumn()) \
                .offset(offset) \
                .limit(count)

        result: LoadPayloadTupleResult = (getTuplesPayloadBlocking(
            self._dbSessionCreator,
            sql,
            self._StateTupleDeclarative.sqlCoreLoad,
            fetchSize=count
        ))

        return result.encodedPayload

    # -------------
    def adlStoreStateInfoTuple(self,
                               item: _StateTupleDeclarative) -> _StateTupleDeclarative:
        """ Store the Info Tuples

        """

        session = self._dbSessionCreator()
        try:
            if item.id is None:
                session.add(item)
            else:
                item = session.merge(item)
            session.commit()
            session.refresh(item)
            session.expunge_all()
            return item

        finally:
            session.close()




from peek_plugin_base.server.PluginServerEntryHookABC import PluginServerEntryHookABC
from peek_plugin_base.client.PluginClientEntryHookABC import PluginClientEntryHookABC
from peek_plugin_base.agent.PluginAgentEntryHookABC import PluginAgentEntryHookABC
from typing import Type

__version__ = '2.3.3'


def peekServerEntryHook() -> Type[PluginServerEntryHookABC]:
    from .private.server.ServerEntryHook import ServerEntryHook
    return ServerEntryHook


def peekAgentEntryHook() -> Type[PluginAgentEntryHookABC]:
    from .private.agent.AgentEntryHook import AgentEntryHook
    return AgentEntryHook
